#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "pff.h"
#include "usart.h"
#include "spi.h"
#include "sd.h"

#define FOREVER 1


FATFS fs;


unsigned int notes_freq[8] =
{
/*DO   RE   MI   FA   SOL  LA   SI   DO*/
261, 294, 329, 349, 391, 440, 466, 523
};


typedef struct {
        char index;
        unsigned int delay;
} note;



/* Time variable used for recording and playing a recorded song */
unsigned int start_time;
/* The placeholder for a recorded song */
note song[256];
/* The length (int notes) of the recorded song */
int song_size;
/* Index of the current note, used for recording and playing */
int crt_note;
/* The current duration of the current note */
unsigned int crt_delay;

/* Variables used for debouncing */
unsigned int control;
unsigned int keyboard;
/* Active notes, used for handling the concurrent pressing of two
 * or more keynotes */
unsigned int active_keyboard;



enum status {DEFAULT, REC, PLAY};

enum status current_mode;


void init() 
{

	current_mode = DEFAULT;


	/* ---INIT BOARD--- */

	/* Set the output pin for buzzer */
	/* timer 1 in fastPWM, prescale 1024 mode */
	DDRD |= (1 << PD5);
	TCCR1A = (1 << WGM10) | (1 << COM1A0);
	TCCR1B = (1 << WGM13) | (1 << CS12);
	OCR1A = 0; //OCR1 A will hold the duration of the note
	/* Set the output pins for LEDs */
	DDRC |= (1 << PC4);
	DDRC |= (1 << PC5);

	/** Set the input pins for the buttons and
	 * activate pull up. 
	 **/
	/* 1. control buttons */
	DDRC &= ~((1 << PC0) | (1 << PC1) | (1 << PC2));
	PORTC |= ((1 << PC0) | (1 << PC1) | (1 << PC2));
	/* 2. keyboard */
	DDRA &= ~((1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | 
		  (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7));
	PORTA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | 
		 (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7);


	/* ---INIT TIMER2--- */

	/* Setting CTC mode */
	TCCR2A = (1 << WGM21);
	/* Setting the prescaler to 1024 */
	TCCR2A |= (1 << CS20) | (1 << CS21) | (1 << CS22);	
	/* Setting the OCR1A where the TOP value will be read from.
	 * Pulses = 156, hence interrupts every 10 ms. 
	 * clock ~= 16MHz, prescaler = 1024 => clock = 15,625kHz
	 * 15,625 kHz => 15 625 pulses per second => 15,6 pulses per milisecond
         * => 156 pulses per 10 miliseconds.
	 */
	OCR2A = 156;
	/* Setting timer interrupts - interrupt on equal with OCR2A */
	TIMSK2 |= (1 << OCIE2A);

}




void setPWM(void)
{
	TCCR1A |= (1 << COM1A0);
}

void unsetPWM(void)
{
	TCCR1A &= ~(1 << COM1A0);
}

void setNote(int note)
{
        /* Choose the specified note */
        OCR1A = notes_freq[note];
}


void check_keys()
{
        int index = -1;
	for (int i = 0; i < 8; ++i)
		if (!(PINA & (1 << i))) {
			/* The button was pressed 10 ms so we mark it
			 * as active (debouncing)*/
			if ((keyboard & (1 << i)) > 0) {
				setNote(i);
				setPWM();
				active_keyboard |= (1 << i);
				keyboard &= ~(1 << i);
				index = i;
			} else {
				keyboard |= (1 << i);
				active_keyboard &= ~(1 << i);
			}
		} else {
			active_keyboard &= ~(1 << i);
			/* If no other key is active we close the
			 * output */
			if ((active_keyboard == 0) && (PINA & (1 << i))) 
				unsetPWM();

		}

}


ISR(TIMER2_COMPA_vect)
{
	

	if (current_mode == DEFAULT) {
		check_keys();		
		
	} else if (current_mode == REC) {
		// save notes into array
		check_keys();
		// write array to the SDcard

	} else {// play
		// read from the SDcard and store into array
		// play the notes from the array
	}
	

	// check what mode button has been pressed
	if (!(PINC & (1 << PC0))) { // default button is pressed
		// turn off the LEDs
		PORTC &= ~((1 << PC5) | (1 << PC4));
		current_mode = DEFAULT;

	} 
	if (!(PINC & (1 << PC1))) { // play button is pressed
		// turn on the green LED
		PORTC |= (1 << PC4);
		// turn off the red LED
		PORTC &= ~(1 << PC5);
		current_mode = PLAY;

	} 
	if (!(PINC & (1 << PC2))) { // rec button is pressed
		// turn on the red LED
		PORTC |= ((1 << PC5));
		// turn off the green LED
		PORTC &= ~(1 << PC4);
		current_mode = REC;
	}
}


int main()
{

	init();
	sei();
	/* wait for interrupts */
	while (FOREVER) {
		if (pf_mount(&fs) != FR_OK) {

			//wait a while and retry
			_delay_ms(1000);
			continue;
		}
	}
	
	return 0;
}

